package com.njyunzhi.pet_identity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.pet_identity.common.BaseController;
import com.njyunzhi.pet_identity.common.Constants;
import com.njyunzhi.pet_identity.common.ResponseBean;
import com.njyunzhi.pet_identity.common.StringUtils;
import com.njyunzhi.pet_identity.entity.SysUser;
import com.njyunzhi.pet_identity.entity.TaAccountLog;
import com.njyunzhi.pet_identity.entity.TaAccountMonthly;
import com.njyunzhi.pet_identity.service.ITaAccountLogService;
import com.njyunzhi.pet_identity.service.ITaAccountMonthlyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

@Api(tags = "账户流水")
@RestController
@RequestMapping("/admin")
public class TaAccountLogController extends BaseController {

    @Autowired
    ITaAccountLogService iTaAccountLogService;

    @Autowired
    ITaAccountMonthlyService iTaAccountMonthlyService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/account-log",method= RequestMethod.GET)
    @ApiOperation(value="账户流水", notes = "账户流水", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDailyList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("日期") @RequestParam(value ="dailyId", required = false) String dailyId,
                                     @ApiParam("月份") @RequestParam(value ="monthId", required = false) String monthId,
                                    @ApiParam("订单号") @RequestParam(value ="orderNo", required = false) String orderNo) throws Exception{

        IPage<TaAccountLog> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaAccountLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(!StringUtils.isEmpty(dailyId), "daily_id", dailyId);
        queryWrapper.like(!StringUtils.isEmpty(orderNo), "order_no", "%" + orderNo + "%");
        queryWrapper.like(!StringUtils.isEmpty(monthId), "daily_id", monthId + "%");
        queryWrapper.orderByDesc("daily_id");
        queryWrapper.orderByDesc("create_date");

        IPage<TaAccountLog> result = iTaAccountLogService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/account-log",method= RequestMethod.POST)
    @ApiOperation(value="对账", notes = "对账", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean newReconciliation(@ApiParam("日期") @RequestParam(value ="dailyId") String dailyId) throws Exception {
        iTaAccountLogService.reconciliation(dailyId);
        return ResponseBean.success("ok");
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/account-log/month",method= RequestMethod.GET)
    @ApiOperation(value="月度账户流水", notes = "月度账户流水", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMonthList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                     @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("月份") @RequestParam(value ="monthId", required = false) String monthId) throws Exception{

        IPage<TaAccountMonthly> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaAccountMonthly> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(!StringUtils.isEmpty(monthId), "month_id", monthId);
        queryWrapper.orderByDesc("month_id");
        queryWrapper.orderByDesc("create_date");

        IPage<TaAccountMonthly> result = iTaAccountMonthlyService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/account-log/month",method= RequestMethod.POST)
    @ApiOperation(value="月度账单生成", notes = "月度账单生成", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean newReconciliationMonth(@ApiParam("月份") @RequestParam(value ="monthId") String monthId) throws Exception {
        iTaAccountLogService.reconcileMonthly(monthId, true);
        return ResponseBean.success("ok");
    }

    @RequestMapping(value="/account-log/month",method= RequestMethod.PUT)
    @ApiOperation(value="月度账单对账", notes = "月度账单对账", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean monthlyUpdate(@ApiParam("月份") @RequestBody TaAccountMonthly taAccountMonthly) throws Exception {
        TaAccountMonthly accountMonthly = iTaAccountMonthlyService.getById(taAccountMonthly.getMonthId());
        if (null == accountMonthly) {
            return ResponseBean.error("未找到对账月度信息");
        }

        accountMonthly.setStatus(taAccountMonthly.getStatus());
        if (Constants.STATUS_NORMAL == taAccountMonthly.getStatus()) {
            accountMonthly.setReconcileDate(LocalDateTime.now());
        }

        iTaAccountMonthlyService.updateById(accountMonthly);
        return ResponseBean.success("ok");
    }
}
