package com.njyunzhi.pet_identity.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.binarywang.wxpay.bean.result.WxPayBillInfo;
import com.github.binarywang.wxpay.bean.result.WxPayBillResult;
import com.njyunzhi.pet_identity.common.Constants;
import com.njyunzhi.pet_identity.common.DateUtils;
import com.njyunzhi.pet_identity.common.WxPayUtils;
import com.njyunzhi.pet_identity.entity.TaAccountLog;
import com.njyunzhi.pet_identity.entity.TaAccountMonthly;
import com.njyunzhi.pet_identity.entity.TaWxBillList;
import com.njyunzhi.pet_identity.mapper.TaAccountLogMapper;
import com.njyunzhi.pet_identity.mapper.TaAccountMonthlyMapper;
import com.njyunzhi.pet_identity.mapper.TaWxBillListMapper;
import com.njyunzhi.pet_identity.service.ITaAccountLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.time.LocalDateTime;

/**
 * <p>
 * 账户流水 服务类
 * </p>
 *
 * @author yansen
 * @since 2022-05-24
 */
@Service
public class TaAccountLogServiceImpl extends BaseServiceImpl<TaAccountLogMapper, TaAccountLog> implements ITaAccountLogService {

    @Autowired
    TaWxBillListMapper taWxBillListMapper;

    @Autowired
    TaAccountMonthlyMapper taAccountMonthlyMapper;

    @Autowired
    WxPayUtils wxPayUtils;

    @Value("${weixin.pay.appId}")
    String payAppId;

    @Override
    public void reconciliation(String dayStr) throws Exception {
        InputStream inputStream = wxPayUtils.download(dayStr);

        // 先删除原始数据
        deleteLogByDailyId(dayStr);

        EasyExcel.read(inputStream, TaWxBillList.class, new PageReadListener<TaWxBillList>(dataList -> {
            if (null != dataList) {
                int len = 0;
                for (TaWxBillList billInfo : dataList) {
                    // 只记录当前 APPID 的
                    String appid = billInfo.getAppId();
                    if (!payAppId.equals(appid)) continue;
                    // 如果读到汇总区, 则代表读完了
                    if ("应结订单总金额".equals(appid)) break;

                    // 先删除原始数据
                    taWxBillListMapper.deleteById(billInfo.getTransactionId());
                    // 再插入
                    taWxBillListMapper.insert(billInfo);

                    TaAccountLog taAccountLog = new TaAccountLog();
                    taAccountLog.setDailyId(dayStr);
                    taAccountLog.setOrderNo(billInfo.getOutTradeNo());

                    // 交易类型
                    int chargeType = "REFUND".equals(billInfo.getTradeState()) ? -1 : 1;
                    taAccountLog.setChargeType(chargeType);
                    // 订单费用
                    int charges = (int) (getDouble(billInfo.getTotalAmount()) * 100);
                    // 手续费 , 微信手续费有正负, 所以不需要依据交易类型特殊处理
                    int poundage = (int) (getDouble(billInfo.getPoundage()) * 100);
                    // 退款金额
                    int appliedRefundAmount = (int) (getDouble(billInfo.getAppliedRefundAmount()) * 100);
                    // 如果是退款
                    if (chargeType == -1) {
                        charges = -appliedRefundAmount;
                    }
                    taAccountLog.setCharges(charges);
                    taAccountLog.setPoundage(poundage);
                    //
                    taAccountLog.setCreateDate(DateUtils.from(billInfo.getTradeTime(), "yyyy-MM-dd HH:mm:ss"));

                    save(taAccountLog);

                    // 记录处理条数
                    len++;
                }

                if (len == 0) {
                    // 如果当月没有明细, 插入一条 0 记录
                    TaAccountLog taAccountLog = new TaAccountLog();
                    taAccountLog.setDailyId(dayStr);
                    taAccountLog.setChargeType(1);
                    taAccountLog.setCharges(0);
                    taAccountLog.setPoundage(0);
                    save(taAccountLog);
                }
            }
        })).excelType(ExcelTypeEnum.CSV).sheet().headRowNumber(1).doRead();
    }

    @Override
    public void reconcileMonthly(String monthStr, boolean force) throws Exception {
        if (!force) {
            if (!checkBillNum(monthStr)) {
                throw new Exception("对账明细不全");
            }
        }

        // 先删除原来的
        TaAccountMonthly origin = taAccountMonthlyMapper.selectById(monthStr);
        if (null != origin) {
            if (Constants.STATUS_NORMAL == origin.getStatus()) {
                throw new Exception(monthStr + " 已经核对过账目");
            }

            taAccountMonthlyMapper.deleteById(monthStr);
        }

        int totalCharge = baseMapper.accTotalCharge(monthStr);
        TaAccountMonthly accountMonthly = new TaAccountMonthly();
        accountMonthly.setMonthId(monthStr);
        accountMonthly.setTotalCharge(totalCharge);
        accountMonthly.setStatus(Constants.STATUS_READY);
        accountMonthly.setCreateDate(LocalDateTime.now());
        taAccountMonthlyMapper.insert(accountMonthly);
    }

    private boolean checkBillNum(String monthStr) {
        String[] parts = monthStr.split("-");
        int days = DateUtils.getDayNum(Integer.parseInt(parts[0]), Integer.parseInt(parts[1]));
        int cnt = baseMapper.countNumMonth(monthStr);

        return days == cnt;
    }

    private double getDouble(String item) {
        if (null == item) return 0.0;

        return Double.parseDouble(item);
    }

    private boolean deleteLogByDailyId(String date) {
        QueryWrapper<TaAccountLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("daily_id", date);
        return remove(queryWrapper);
    }
}
